@@ -1,9 +1,10 @@
-# Copyright 2001-4 by Roger Bivand
+# Copyright 2001-9 by Roger Bivand
 #
 
 
 tri2nb <- function(coords, row.names = NULL) {
-	require("tripack")
+#	require("tripack")
+	require("deldir")
 	n <- nrow(coords)
 	if (n < 3) stop("too few coordinates")
 #	left <- function(x) {
@@ -19,8 +20,16 @@ tri2nb <- function(coords, row.names = NULL) {
 	    		stop("non-unique row.names given")
     	}
     	if (is.null(row.names)) row.names <- as.character(1:n)
-	tri <- tri.mesh(x=coords[,1], y=coords[,2])
-	nb <- neighbours(tri)
+#	tri <- tri.mesh(x=coords[,1], y=coords[,2])
+        tri <- deldir(x=coords[,1], y=coords[,2])
+        from <- c(tri$delsgs[,5], tri$delsgs[,6])
+        to <- c(tri$delsgs[,6], tri$delsgs[,5])
+        df <- data.frame(from=as.integer(from), to=as.integer(to), weight=1)
+        attr(df, "n") <- tri$n.data
+        class(df) <- c(class(df), "spatial.neighbour")
+        df1 <- df[order(df$from),]
+        nb <- sn2listw(df1)$neighbours
+#	nb <- neighbours(tri)
  	attr(nb, "region.id") <- row.names
 	class(nb) <- "nb"
 	attr(nb, "tri") <- TRUE
